class InfAPRocketLauncher : DoomWeapon {
	Default {
		//$Category Weapons
		Weapon.SelectionOrder 2400;
		Weapon.AmmoUse 1;
		Weapon.AmmoUse2 1;
		Weapon.AmmoGive 2;
		Weapon.AmmoType "RocketAmmo";
		Weapon.AmmoType2 "RocketAmmo";
		Inventory.PickupMessage "You got the dual-mode Rocket Launcher!";
		Tag "AP Rocket Launcher";
		Obituary "%o was destroyed by %k's AP rocket launcher.";
	}
	States {
		Ready:
			MI2G A 1 A_WeaponReady;
			Loop;
		Deselect:
			MI2G A 1 A_Lower;
			Loop;
		Select:
			MI2G A 1 A_Raise;
			Loop;
		Fire:
			TNT1 A 0 A_JumpIf(GetCvar("cl_infested_swap_aprl_firemodes"), "AltFireModeSwap");
		FireModeSwap:
			MI2G B 8 A_GunFlash;
			MI2G C 4 A_FireProjectile("InfAPRocket");
			MI2G D 4;
			MI2G C 4;
			MI2G B 0 A_ReFire;
			Goto Ready;
		AltFire:
			TNT1 A 0 A_JumpIf(GetCvar("cl_infested_swap_aprl_firemodes"), "FireModeSwap");
		AltFireModeSwap:
			MI2G B 8 A_GunFlash;
			MI2G C 4 A_FireMissile;
			MI2G D 4;
			MI2G C 4;
			MI2G B 0 A_ReFire;
			Goto Ready;
		Flash:
			MI2F A 3 Bright A_Light1;
			MI2F B 4 Bright;
			MI2F C 4 Bright A_Light2;
			Goto LightDone;
		Flash:
			MI2F A 3 Bright A_Light1;
			MI2F B 4 Bright;
			MI2F C 4 Bright A_Light2;
			Goto LightDone;
		Spawn:
			LAU2 A -1;
			Stop;
	}
}

class InfAPRocket : Rocket {
    Array<Actor> ShotList;		//To make sure a monster can only be damaged once
	bool HitActor;
	bool Detonated;

	Default	{
		Damage 1;
		BounceType "Hexen";
		BounceFactor 1.0;
		WallBounceFactor 1.0;
		ReactionTime 16;		//After detonation, the piercing projectile disappears after 16 tics or less
		SeeSound "weapons/aprlfire";
		DeathSound "";
		Translation "128:151=112:127";
		+USEBOUNCESTATE
		+DONTBOUNCEONSKY
		+CANNOTPUSH
		+NODAMAGETHRUST
	}
	Override int SpecialMissileHit(Actor victim){	//Returning 1 pierces, returning 0 destroys projectile
		if(ShotList.Find(victim) == ShotList.Size()) {
			if(victim == Self.target){
				if(Detonated == TRUE){		//Only damage self if the charge has detonated
					victim.DamageMobj(Self, target, 50, "Bullet");
					ShotList.Push(victim);
				}
				return 1;
			} else {
				if(HitActor == TRUE || Detonated == TRUE){
					// Subsequent monsters are hit for 80-120 damage by the piercing charge
					victim.DamageMobj(Self, target, 20*random(1,3)+60, "Bullet");
				} else {
					// First monster is hit for 110-250 damage by the piercing charge (plus ~40 splash damage from direct rocket hit)
					victim.DamageMobj(Self, target, 20*random(1,8)+90, "Bullet");
				}
				ShotList.Push(victim);	//Add monster to the list so it won't be damaged again
				HitActor = TRUE;
                if(victim.bSHOOTABLE && !victim.bNOBLOOD && !victim.bSPECTRAL){
					victim.SpawnBlood(self.pos,self.angle,200);		//If it can bleed, make it bleed
				}
                if(victim.bISMONSTER && !victim.bNOBLOOD){
					victim.target==self.target; victim.A_AlertMonsters(1);		//Alert it too
				}
				return 1;
            }
        } else {
			return 1;
		}
	}
	States {
	Spawn:
		TNT1 A 0 A_JumpIf(HitActor == TRUE, "Bounce");
		MISL A 1 Bright;
		Loop;
	Bounce:
		TNT1 A 0 {
			if(Detonated == FALSE){
				Detonated = TRUE;
				A_ScaleVelocity(2.5);
				A_StartSound("weapons/rocklx", CHAN_AUTO);
				A_SpawnItem("APRocketExplosion");
				A_Explode(40, 64);	// Smol explosion - but enough to discourage facerockets
			}
		}
		TNT1 A 0 A_Countdown();
		SCTR L 1 Bright	A_SpawnItem("ScrapTrail");
		TNT1 A 0 A_Countdown();
		SCTR M 1 Bright	A_SpawnItem("ScrapTrail");
		Loop;
	Death:
		SCTR NOPQRSTUVWX 1 Bright;
		Stop;
	}
}

class ScrapTrail : Actor {
	Default	{
		Radius 1;
		Height 1;
		Speed 0;
		Alpha 0.6;
		+NOINTERACTION
		+CLIENTSIDEONLY
		+RANDOMIZE
	}
    States {
		Spawn:
			SCTR NOPQRSTUVWX 1 Bright;
			Stop;
    }
}

class APRocketExplosion : Actor {
	Default	{
		Radius 1;
		Height 1;
		Speed 0;
		Alpha 0.6;
		+NOINTERACTION
		+CLIENTSIDEONLY
		+RANDOMIZE
	}
    States {
		Spawn:
			MISL B 2 Bright;
			MISL C 2 Bright;
			MISL D 2 Bright;
			Stop;
    }
}